/*
 * ProFTPD - FTP server daemon
 * Copyright (c) 1997, 1998 Public Flood Software
 * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
 * Copyright (c) 2001-2007 The ProFTPD Project team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * As a special exemption, Public Flood Software/MacGyver aka Habeeb J. Dihu
 * and other respective copyright holders give permission to link this program
 * with OpenSSL, and distribute the resulting executable, without including
 * the source code for OpenSSL in the source distribution.
 */

/*
 * Wu-FTPd style Anonymous Authentication password type check module for ProFTPD
 * Copyright (c) JoungKyun.Kim <http://oops.org>
 * $Id: mod_anontype.c,v 1.1.1.1 2007/12/07 15:54:46 oops Exp $
 */

#include "conf.h"
#include "privs.h"

#ifdef HAVE_REGEX_H
#include <regex.h>
#define REG_TRIVIAL "@"
#define REG_COMPLETE "^[a-z0-9_-]+@[a-z0-9._-]+\\.([a-z]{2}|com|net|org|biz|info|mobi)$"
#endif /* HAVE_REGEX_H */

#define MOD_ANONTYPE_VERSION		"mod_anontype/0.1"

module anontype_module;

#define ANONTYPE_TRIVIAL 0
#define ANONTYPE_COMPLETE 1
#define ANONTYPE_FAILED 100

/* Declaration of command handler */
MODRET set_anonpasstype (cmd_rec *cmd) {
	unsigned char *passtype = NULL;

	CHECK_ARGS (cmd, 1);
	CHECK_CONF (cmd, CONF_ANON);

	if ( ! strcasecmp (cmd->argv[1], "none") )
		passtype = "none";
	else {
		if ( ! strcasecmp (cmd->argv[1], "trivial") ||
			 ! strcasecmp (cmd->argv[1], "complete-email") )
			passtype = cmd->argv[1];
		else
			passtype = "none";
	}

	add_config_param_str (cmd->argv[0], 1, passtype);

	return PR_HANDLED (cmd);
}

MODRET set_anonpasstype_debug (cmd_rec *cmd) {
	int bool = -1;
	config_rec *c = NULL;

	CHECK_ARGS (cmd, 1);
	CHECK_CONF (cmd, CONF_ROOT|CONF_GLOBAL|CONF_ANON);

	bool = get_boolean (cmd, 1);
	if ( bool == -1 )
		CONF_ERROR (cmd, _("expected Boolean parameter"));

	c = add_config_param(cmd->argv[0], 1, NULL);
	c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
	*((unsigned char *) c->argv[0]) = bool;

	return PR_HANDLED (cmd);
}

MODRET anontype_pass_check (cmd_rec *cmd) {
#if defined (HAVE_REGEX_H) && defined (HAVE_REGCOMP)
	config_rec * ac_value, * ac_debug, * ac_req;
	short debug_flag = 0;

	/* No anonymous mode */
	if ( session.anon_user == NULL )
		return 0;

	/* set AnonRequirePassword, skip */
	ac_req = get_param_ptr (session.anon_config->subset, "AnonRequirePassword", FALSE);
	if ( ac_req && * (unsigned char *) ac_req == TRUE )
		return 0;

	ac_value = get_param_ptr (session.anon_config->subset, "AnonPassType", FALSE);

	/* get Debug flag */
	ac_debug = get_param_ptr (TOPLEVEL_CONF, "AnonPassTypeDebug", FALSE);
	if ( ! ac_debug )
		ac_debug = get_param_ptr (main_server->conf, "AnonPassTypeDebug", FALSE);

	if ( ac_debug && * (unsigned char *) ac_debug == TRUE )
		debug_flag = 1;

	/*
	if ( debug_flag )
		log_auth (LOG_NOTICE, "AnonPassType Directive Value: %s", ac_value);
	*/

	/*
	 * AnonPassType vlue
	 *  NULL => directive not set
	 *  none => don't check
	 */
	if ( ac_value == NULL || ! strcasecmp ((unsigned char *) ac_value, "none") )
		return 0;

	{
		regex_t anontype_reg;
		unsigned char * anontype_reg_value;
		unsigned char * pass;
		short anon_mode;

		pass = cmd->arg;

		if ( ! strcasecmp ((unsigned char *) ac_value , "trivial") ) {
			anontype_reg_value = REG_TRIVIAL;
			anon_mode = ANONTYPE_TRIVIAL;
		//} else if ( ! strcasecmp ((unsigned char *) ac_value , "complete-email") ) {
		} else {
			anontype_reg_value = REG_COMPLETE;
			anon_mode = ANONTYPE_COMPLETE;
		}

		if ( regcomp (&anontype_reg, (char *) anontype_reg_value, REG_EXTENDED | REG_ICASE) != 0) {
			/* failed regex compile, so auth success */
			return 0;
		}

		if ( regexec (&anontype_reg, pass, 0, NULL, 0) != 0 ) {
			log_auth (LOG_NOTICE, _("%s: ANON %s: Password format (%s) invalid '%s'"),
					MOD_ANONTYPE_VERSION, session.user, ac_value, pass);

			if ( anon_mode == ANONTYPE_TRIVIAL )
				pr_response_send (R_530,
						_("Login failed: password is not included '@' character.\n"));
			else
				pr_response_send(R_530,
						_("Login failed: `%s` is not invalid email address\n"), pass);

			anon_mode = ANONTYPE_FAILED;
		}
		regfree (&anontype_reg);

		if ( anon_mode == ANONTYPE_FAILED ) {
			log_auth (LOG_NOTICE, _("%s: ANON %s: Login refused."),
					MOD_ANONTYPE_VERSION, session.user);
			pr_response_send(R_530, _("Login incorrect."));
			end_login (1);
		}
	}
#endif

	return PR_DECLINED (cmd);
}

static conftable anontype_conftab[] = {
	{ "AnonPassType",      set_anonpasstype,       NULL },
	{ "AnonPassTypeDebug", set_anonpasstype_debug, NULL },
	{ 0,                   NULL,                   NULL }
};

static cmdtable anontype_cmdtab[] = {
	{ POST_CMD, C_PASS, G_NONE, anontype_pass_check, FALSE, FALSE, CL_AUTH },
	{ 0, NULL }
};

/* Module interface */

module anontype_module = {
	NULL, NULL,	            /* Always NULL */
	0x20,                   /* Module API version (2.0) */
	"anontype",             /* Module name */
	anontype_conftab,       /* Module configuration directive table */
	anontype_cmdtab,        /* Module command handler table */
	NULL,                   /* Module authentication handler table */
	NULL,                   /* Module initialization function */
	NULL,                   /* Session initialization function */
	MOD_ANONTYPE_VERSION    /* Module version */
};
